#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/ioctl.h>
#include <signal.h>

#ifndef	__user
#define __user
#endif

#include "vstusb.h"

#define OFF	0
#define ON	1

#define EP2  2
#define EP7  7

#define NDR_TIMEOUT     5

//globals
char	*TestFileName;

//bob can handle it!
void sig_bob(int signo) 
{
		//this is the signal handler
		printf("%s: signal (%d) handled\n", __func__, signo);

}

void reader(void *arg)
{
	int dev = *(int*)arg;
	int retval;
	char readBuf[4096]="";
	pid_t pid = getpid();
	int pri;
	sigset_t sigset;
	struct sigaction sigact;

	int i;


	//UNBLOCK all signals
	sigfillset(&sigset);
	pthread_sigmask(SIG_SETMASK, &sigset, NULL);
	pthread_sigmask(SIG_UNBLOCK, &sigset, NULL);

	//SET ALL TO bob SIGACTION
	sigact.sa_flags = 0; //SA_RESTART has no effect;
	sigact.sa_handler = sig_bob; //if you can't handle it, make bob do it.
	sigact.sa_mask = sigset;
	sigact.sa_restorer = 0;

	for (i=5; i<NSIG; i++) {
			sigaction(i, &sigact, NULL);
	}

	//clear some vars
	memset(readBuf, 0, sizeof(readBuf));
	errno = 0;

/* set the priority of the thread, has no effect? */
	pri = getpriority(PRIO_PROCESS, 0);
	if (errno != 0) {
		printf("%s: getpriority error: %s\n", __FUNCTION__, strerror(errno));
		return;
	}
	
	printf("%s: dev = %d  pid=%d   pri=%d\n", __FUNCTION__, dev, pid, pri);

	if ((retval = setpriority(PRIO_PROCESS, 0, 10)) != 0) {
		printf("%s: setpriority error (%d): %s\n", __FUNCTION__, retval, strerror(errno));
		return;
	} 
	errno = 0;
	pri = getpriority(PRIO_PROCESS, 0);
	if (errno != 0) {
		printf("%s: getpriority error: %s\n", __FUNCTION__, strerror(errno));
		return;
	}

	printf("%s: dev = %d  pid=%d   pri=%d\n", __FUNCTION__, dev, pid, pri);
/* done set priority */


	while (1) {
		//printf("%s: do read\n", __FUNCTION__);
#if 0
		usb_data.timeout_ms = 0;
		usb_data.buffer = (void *)readBuf;
		usb_data.count = 4096;
		usb_data.pipe = 1;
		if ((retval = ioctl((int)dev, IOCTL_VSTUSB_RECV_PIPE, &usb_data)) != 0) {
			printf("%s: no can read, error is %s\n", __FUNCTION__,strerror(errno));
			goto exit;
		}
#else
		memset(readBuf, 0, 4096);
		if ((retval = read(dev, readBuf, 4096)) <= 0) {
			printf("%s: no can read (retval= %d), error is %s (%d)\n", __FUNCTION__, retval, strerror(errno), errno);
			//break;
		}
#endif
		if (retval > 0) {
				readBuf[retval]='\0';
				//printf("%s: received (%d): %s\n", __FUNCTION__, retval, readBuf);
		}
	}
	printf("%s: exiting (%d)\n", __FUNCTION__, retval);
}

int parse_cmdline (argc, argv)
{
	if (argc != 2) {
//		printf("invalid number of arguments to test program");
//		return -1;
	}

	TestFileName = NULL;

	return 0;
}

int main(int argc, char *argv[])
{
	int size;
	int i, retval=0;
	struct vstusb_args	usb_data;
	int writeFlag=1;
	char writeBuf[4096]="nowisthetime";
	int dev = -1;			//device file pointer
    int threadRetVal;
	void *threadRetValp = &threadRetVal;

	if (parse_cmdline(argc, argv) != 0)
		return -1;

	if ((dev = open("/dev/vstusb0", O_RDWR)) == -1) {
		printf("%s: can't open /dev/vstusb0 (%d), errno = %s\n", __FUNCTION__, dev, strerror(errno));
		return dev;
	}

    	usb_data.rd_pipe = 1;
    	usb_data.rd_timeout_ms = 0;
    	usb_data.wr_pipe = 2;
    	usb_data.wr_timeout_ms = 1000;
	if ((retval = ioctl((int)dev, IOCTL_VSTUSB_CONFIG_RW, &usb_data)) != 0) {
		printf("%s: no can write, error is %s\n", __FUNCTION__,strerror(errno));
		goto exit;
	}

	pthread_t thread;
	if ((retval = pthread_create(&thread, NULL, (void* (*)(void*))reader, (void*)&dev)) != 0) {
		printf("%s: pthread_create fail (%d), errno = %s\n", __FUNCTION__, retval, strerror(errno));
		goto exit;
	}

	printf("%s: START\n", __FUNCTION__);
	for (i=0; i<10; i++) {
	//while (1) {
		printf("%s: do write\n", __FUNCTION__);
#if 0
		usb_data.timeout_ms = 100000;
		usb_data.buffer = (void *)writeBuf;
		usb_data.count = 13;
		usb_data.pipe = 2;
		if ((retval = ioctl((int)dev, IOCTL_VSTUSB_SEND_PIPE, &usb_data)) != 0) {
			printf("%s: no can write, error is %s\n", __FUNCTION__,strerror(errno));
			goto exit;
		}
#else
		if ((retval = write((int)dev, writeBuf, 13)) != 13) {
			printf("%s: no can write, error is %s\n", __FUNCTION__,strerror(errno));
			break;
		} 
		printf("trykill\n");
		retval = pthread_kill(thread, SIGUSR1);
		if (retval != 0) {
				printf("%s: pthread_kill returned %08x : %s\n", __FUNCTION__, retval, strerror(retval));
		}
		sleep(1);
#endif
	}

exit:

	printf("trykill\n");
	retval = pthread_kill(thread, SIGUSR1);
	if (retval != 0) {
			printf("%s: pthread_kill returned %08x : %s\n", __FUNCTION__, retval, strerror(retval));
	}
	printf("sleep 5\n");
	sleep(5);
	/**/

	printf("%s: call pthread_cancel on thread %08x\n", __FUNCTION__, thread);
	retval = pthread_cancel(thread);
	if (retval != 0) {
			printf("%s: pthread_cancel returned %08x : %s\n", __FUNCTION__, retval, strerror(retval));
	}
	printf("sleep 5\n");
	sleep(5);

	printf("%s: call pthread_join on thread %08x\n", __FUNCTION__, thread);
	retval = pthread_join(thread, &threadRetValp);
	if (retval != 0) {
			printf("%s: pthread_join returned %08x : %s\n", __FUNCTION__, retval, strerror(retval));
	}
	else {
			printf("%s: pthread_join returned 0, cancelled thread returned %08x : %s\n", __FUNCTION__, threadRetValp, strerror(retval));
	}

	printf("%s: thread gone\n", __FUNCTION__);

	if (close(dev) < 0) {
		printf("%s: close err /dev/vstusb (%d)", __FUNCTION__, retval);
	}
	printf("%s: closed\n", __FUNCTION__);

	return 0;
}
